@@ -4,12 +4,15 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "nsHTMLDNSPrefetch.h"
+
 #include "base/basictypes.h"
+#include "mozilla/dom/Element.h"
 #include "mozilla/net/NeckoCommon.h"
 #include "mozilla/net/NeckoChild.h"
+#include "mozilla/StoragePrincipalHelper.h"
 #include "nsURLHelper.h"
 
-#include "nsHTMLDNSPrefetch.h"
 #include "nsCOMPtr.h"
 #include "nsString.h"
 
@@ -32,18 +35,17 @@
 
 #include "mozilla/Components.h"
 #include "mozilla/Preferences.h"
+#include "mozilla/StaticPrefs_network.h"
 
 using namespace mozilla;
 using namespace mozilla::dom;
 using namespace mozilla::net;
 
 static NS_DEFINE_CID(kDNSServiceCID, NS_DNSSERVICE_CID);
-bool sDisablePrefetchHTTPSPref;
 static bool sInitialized = false;
 static nsIDNSService* sDNSService = nullptr;
 static nsHTMLDNSPrefetch::nsDeferrals* sPrefetches = nullptr;
 static nsHTMLDNSPrefetch::nsListener* sDNSListener = nullptr;
-bool sEsniEnabled;
 
 nsresult nsHTMLDNSPrefetch::Initialize() {
   if (sInitialized) {
@@ -59,22 +61,6 @@ nsresult nsHTMLDNSPrefetch::Initialize() {
 
   sPrefetches->Activate();
 
-  Preferences::AddBoolVarCache(&sDisablePrefetchHTTPSPref,
-                               "network.dns.disablePrefetchFromHTTPS");
-
-  Preferences::AddBoolVarCache(&sEsniEnabled, "network.security.esni.enabled");
-
-  // Default is false, so we need an explicit call to prime the cache.
-  sDisablePrefetchHTTPSPref =
-      Preferences::GetBool("network.dns.disablePrefetchFromHTTPS", true);
-
-  sEsniEnabled = Preferences::GetBool("network.security.esni.enabled", false);
-
-  NS_IF_RELEASE(sDNSService);
-  nsresult rv;
-  rv = CallGetService(kDNSServiceCID, &sDNSService);
-  if (NS_FAILED(rv)) return rv;
-
   if (IsNeckoChild()) NeckoChild::InitNeckoChild();
 
   sInitialized = true;
@@ -94,15 +80,41 @@ nsresult nsHTMLDNSPrefetch::Shutdown() {
   return NS_OK;
 }
 
+static bool EnsureDNSService() {
+  if (sDNSService) {
+    return true;
+  }
+
+  NS_IF_RELEASE(sDNSService);
+  nsresult rv;
+  rv = CallGetService(kDNSServiceCID, &sDNSService);
+  if (NS_FAILED(rv)) {
+    return false;
+  }
+
+  return !!sDNSService;
+}
+
 bool nsHTMLDNSPrefetch::IsAllowed(Document* aDocument) {
   // There is no need to do prefetch on non UI scenarios such as XMLHttpRequest.
   return aDocument->IsDNSPrefetchAllowed() && aDocument->GetWindow();
 }
 
-nsresult nsHTMLDNSPrefetch::Prefetch(Link* aElement, uint16_t flags) {
-  if (!(sInitialized && sPrefetches && sDNSService && sDNSListener))
+static uint32_t GetDNSFlagsFromLink(Link* aElement) {
+  if (!aElement || !aElement->GetElement() ||
+      !aElement->GetElement()->OwnerDoc()->GetChannel()) {
+    return 0;
+  }
+  nsIRequest::TRRMode mode =
+      aElement->GetElement()->OwnerDoc()->GetChannel()->GetTRRMode();
+  return nsIDNSService::GetFlagsFromTRRMode(mode);
+}
+
+nsresult nsHTMLDNSPrefetch::Prefetch(Link* aElement, uint32_t flags) {
+  if (!(sInitialized && sPrefetches && sDNSListener) || !EnsureDNSService())
     return NS_ERROR_NOT_AVAILABLE;
 
+  flags |= GetDNSFlagsFromLink(aElement);
   return sPrefetches->Add(flags, aElement);
 }
 
@@ -118,9 +130,10 @@ nsresult nsHTMLDNSPrefetch::PrefetchHigh(Link* aElement) {
   return Prefetch(aElement, 0);
 }
 
-nsresult nsHTMLDNSPrefetch::Prefetch(const nsAString& hostname, bool isHttps,
-                                     const OriginAttributes& aOriginAttributes,
-                                     uint16_t flags) {
+nsresult nsHTMLDNSPrefetch::Prefetch(
+    const nsAString& hostname, bool isHttps,
+    const OriginAttributes& aPartitionedPrincipalOriginAttributes,
+    uint32_t flags) {
   if (IsNeckoChild()) {
     // We need to check IsEmpty() because net_IsValidHostName()
     // considers empty strings to be valid hostnames
@@ -129,32 +142,34 @@ nsresult nsHTMLDNSPrefetch::Prefetch(const nsAString& hostname, bool isHttps,
       // during shutdown gNeckoChild might be null
       if (gNeckoChild) {
         gNeckoChild->SendHTMLDNSPrefetch(nsString(hostname), isHttps,
-                                         aOriginAttributes, flags);
+                                         aPartitionedPrincipalOriginAttributes,
+                                         flags);
       }
     }
     return NS_OK;
   }
 
-  if (!(sInitialized && sDNSService && sPrefetches && sDNSListener))
+  if (!(sInitialized && sPrefetches && sDNSListener) || !EnsureDNSService())
     return NS_ERROR_NOT_AVAILABLE;
 
   nsCOMPtr<nsICancelable> tmpOutstanding;
   nsresult rv = sDNSService->AsyncResolveNative(
       NS_ConvertUTF16toUTF8(hostname), flags | nsIDNSService::RESOLVE_SPECULATE,
-      sDNSListener, nullptr, aOriginAttributes, getter_AddRefs(tmpOutstanding));
+      sDNSListener, nullptr, aPartitionedPrincipalOriginAttributes,
+      getter_AddRefs(tmpOutstanding));
   if (NS_FAILED(rv)) {
     return rv;
   }
 
   // Fetch ESNI keys if needed.
-  if (isHttps && sEsniEnabled) {
+  if (isHttps && StaticPrefs::network_security_esni_enabled()) {
     nsAutoCString esniHost;
     esniHost.Append("_esni.");
     esniHost.Append(NS_ConvertUTF16toUTF8(hostname));
     Unused << sDNSService->AsyncResolveByTypeNative(
         esniHost, nsIDNSService::RESOLVE_TYPE_TXT,
         flags | nsIDNSService::RESOLVE_SPECULATE, sDNSListener, nullptr,
-        aOriginAttributes, getter_AddRefs(tmpOutstanding));
+        aPartitionedPrincipalOriginAttributes, getter_AddRefs(tmpOutstanding));
   }
 
   return NS_OK;
@@ -162,27 +177,33 @@ nsresult nsHTMLDNSPrefetch::Prefetch(const nsAString& hostname, bool isHttps,
 
 nsresult nsHTMLDNSPrefetch::PrefetchLow(
     const nsAString& hostname, bool isHttps,
-    const OriginAttributes& aOriginAttributes) {
-  return Prefetch(hostname, isHttps, aOriginAttributes,
-                  nsIDNSService::RESOLVE_PRIORITY_LOW);
+    const OriginAttributes& aPartitionedPrincipalOriginAttributes,
+    nsIRequest::TRRMode aMode) {
+  return Prefetch(hostname, isHttps, aPartitionedPrincipalOriginAttributes,
+                  nsIDNSService::GetFlagsFromTRRMode(aMode) |
+                      nsIDNSService::RESOLVE_PRIORITY_LOW);
 }
 
 nsresult nsHTMLDNSPrefetch::PrefetchMedium(
     const nsAString& hostname, bool isHttps,
-    const OriginAttributes& aOriginAttributes) {
-  return Prefetch(hostname, isHttps, aOriginAttributes,
-                  nsIDNSService::RESOLVE_PRIORITY_MEDIUM);
+    const OriginAttributes& aPartitionedPrincipalOriginAttributes,
+    nsIRequest::TRRMode aMode) {
+  return Prefetch(hostname, isHttps, aPartitionedPrincipalOriginAttributes,
+                  nsIDNSService::GetFlagsFromTRRMode(aMode) |
+                      nsIDNSService::RESOLVE_PRIORITY_MEDIUM);
 }
 
 nsresult nsHTMLDNSPrefetch::PrefetchHigh(
     const nsAString& hostname, bool isHttps,
-    const OriginAttributes& aOriginAttributes) {
-  return Prefetch(hostname, isHttps, aOriginAttributes, 0);
+    const OriginAttributes& aPartitionedPrincipalOriginAttributes,
+    nsIRequest::TRRMode aMode) {
+  return Prefetch(hostname, isHttps, aPartitionedPrincipalOriginAttributes,
+                  nsIDNSService::GetFlagsFromTRRMode(aMode));
 }
 
-nsresult nsHTMLDNSPrefetch::CancelPrefetch(Link* aElement, uint16_t flags,
+nsresult nsHTMLDNSPrefetch::CancelPrefetch(Link* aElement, uint32_t flags,
                                            nsresult aReason) {
-  if (!(sInitialized && sPrefetches && sDNSService && sDNSListener))
+  if (!(sInitialized && sPrefetches && sDNSListener) || !EnsureDNSService())
     return NS_ERROR_NOT_AVAILABLE;
 
   nsAutoString hostname;
@@ -197,15 +218,18 @@ nsresult nsHTMLDNSPrefetch::CancelPrefetch(Link* aElement, uint16_t flags,
   if (protocol.EqualsLiteral("https:")) {
     isHttps = true;
   }
-  return CancelPrefetch(hostname, isHttps,
-                        element->NodePrincipal()->OriginAttributesRef(), flags,
-                        aReason);
+
+  OriginAttributes oa;
+  StoragePrincipalHelper::GetOriginAttributesForNetworkState(
+      element->OwnerDoc(), oa);
+
+  return CancelPrefetch(hostname, isHttps, oa, flags, aReason);
 }
 
 nsresult nsHTMLDNSPrefetch::CancelPrefetch(
     const nsAString& hostname, bool isHttps,
-    const OriginAttributes& aOriginAttributes, uint16_t flags,
-    nsresult aReason) {
+    const OriginAttributes& aPartitionedPrincipalOriginAttributes,
+    uint32_t flags, nsresult aReason) {
   // Forward this request to Necko Parent if we're a child process
   if (IsNeckoChild()) {
     // We need to check IsEmpty() because net_IsValidHostName()
@@ -215,42 +239,50 @@ nsresult nsHTMLDNSPrefetch::CancelPrefetch(
       // during shutdown gNeckoChild might be null
       if (gNeckoChild) {
         gNeckoChild->SendCancelHTMLDNSPrefetch(
-            nsString(hostname), isHttps, aOriginAttributes, flags, aReason);
+            nsString(hostname), isHttps, aPartitionedPrincipalOriginAttributes,
+            flags, aReason);
       }
     }
     return NS_OK;
   }
 
-  if (!(sInitialized && sDNSService && sPrefetches && sDNSListener))
+  if (!(sInitialized && sPrefetches && sDNSListener) || !EnsureDNSService())
     return NS_ERROR_NOT_AVAILABLE;
 
   // Forward cancellation to DNS service
   nsresult rv = sDNSService->CancelAsyncResolveNative(
       NS_ConvertUTF16toUTF8(hostname), flags | nsIDNSService::RESOLVE_SPECULATE,
-      sDNSListener, aReason, aOriginAttributes);
+      sDNSListener, aReason, aPartitionedPrincipalOriginAttributes);
   // Cancel fetching ESNI keys if needed.
-  if (sEsniEnabled && isHttps) {
+  if (StaticPrefs::network_security_esni_enabled() && isHttps) {
     nsAutoCString esniHost;
     esniHost.Append("_esni.");
     esniHost.Append(NS_ConvertUTF16toUTF8(hostname));
     sDNSService->CancelAsyncResolveByTypeNative(
         esniHost, nsIDNSService::RESOLVE_TYPE_TXT,
         flags | nsIDNSService::RESOLVE_SPECULATE, sDNSListener, aReason,
-        aOriginAttributes);
+        aPartitionedPrincipalOriginAttributes);
   }
   return rv;
 }
 
 nsresult nsHTMLDNSPrefetch::CancelPrefetchLow(Link* aElement,
                                               nsresult aReason) {
-  return CancelPrefetch(aElement, nsIDNSService::RESOLVE_PRIORITY_LOW, aReason);
+  return CancelPrefetch(
+      aElement,
+      GetDNSFlagsFromLink(aElement) | nsIDNSService::RESOLVE_PRIORITY_LOW,
+      aReason);
 }
 
 nsresult nsHTMLDNSPrefetch::CancelPrefetchLow(
     const nsAString& hostname, bool isHttps,
-    const OriginAttributes& aOriginAttributes, nsresult aReason) {
-  return CancelPrefetch(hostname, isHttps, aOriginAttributes,
-                        nsIDNSService::RESOLVE_PRIORITY_LOW, aReason);
+    const OriginAttributes& aPartitionedPrincipalOriginAttributes,
+    nsIRequest::TRRMode aTRRMode, nsresult aReason) {
+  return CancelPrefetch(hostname, isHttps,
+                        aPartitionedPrincipalOriginAttributes,
+                        nsIDNSService::GetFlagsFromTRRMode(aTRRMode) |
+                            nsIDNSService::RESOLVE_PRIORITY_LOW,
+                        aReason);
 }
 
 void nsHTMLDNSPrefetch::LinkDestroyed(Link* aLink) {
@@ -272,13 +304,6 @@ nsHTMLDNSPrefetch::nsListener::OnLookupComplete(nsICancelable* request,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsHTMLDNSPrefetch::nsListener::OnLookupByTypeComplete(nsICancelable* request,
-                                                      nsIDNSByTypeRecord* res,
-                                                      nsresult status) {
-  return NS_OK;
-}
-
 /////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 nsHTMLDNSPrefetch::nsDeferrals::nsDeferrals()
@@ -309,7 +334,7 @@ void nsHTMLDNSPrefetch::nsDeferrals::Flush() {
   }
 }
 
-nsresult nsHTMLDNSPrefetch::nsDeferrals::Add(uint16_t flags, Link* aElement) {
+nsresult nsHTMLDNSPrefetch::nsDeferrals::Add(uint32_t flags, Link* aElement) {
   // The FIFO has no lock, so it can only be accessed on main thread
   NS_ASSERTION(NS_IsMainThread(), "nsDeferrals::Add must be on main thread");
 
@@ -336,7 +361,9 @@ void nsHTMLDNSPrefetch::nsDeferrals::SubmitQueue() {
   NS_ASSERTION(NS_IsMainThread(),
                "nsDeferrals::SubmitQueue must be on main thread");
   nsCString hostName;
-  if (!sDNSService) return;
+  if (!EnsureDNSService()) {
+    return;
+  }
 
   while (mHead != mTail) {
     nsCOMPtr<Link> link = mEntries[mTail].mElement;
@@ -356,19 +383,21 @@ void nsHTMLDNSPrefetch::nsDeferrals::SubmitQueue() {
           rv = NS_URIChainHasFlags(hrefURI,
                                    nsIProtocolHandler::URI_IS_LOCAL_RESOURCE,
                                    &isLocalResource);
-
-          hrefURI->SchemeIs("https", &isHttps);
+          isHttps = hrefURI->SchemeIs("https");
         }
 
+        OriginAttributes oa;
+        StoragePrincipalHelper::GetOriginAttributesForNetworkState(
+            element->OwnerDoc(), oa);
+
         if (!hostName.IsEmpty() && NS_SUCCEEDED(rv) && !isLocalResource &&
             element) {
           if (IsNeckoChild()) {
             // during shutdown gNeckoChild might be null
             if (gNeckoChild) {
-              gNeckoChild->SendHTMLDNSPrefetch(
-                  NS_ConvertUTF8toUTF16(hostName), isHttps,
-                  element->NodePrincipal()->OriginAttributesRef(),
-                  mEntries[mTail].mFlags);
+              gNeckoChild->SendHTMLDNSPrefetch(NS_ConvertUTF8toUTF16(hostName),
+                                               isHttps, oa,
+                                               mEntries[mTail].mFlags);
             }
           } else {
             nsCOMPtr<nsICancelable> tmpOutstanding;
@@ -376,20 +405,17 @@ void nsHTMLDNSPrefetch::nsDeferrals::SubmitQueue() {
             rv = sDNSService->AsyncResolveNative(
                 hostName,
                 mEntries[mTail].mFlags | nsIDNSService::RESOLVE_SPECULATE,
-                sDNSListener, nullptr,
-                element->NodePrincipal()->OriginAttributesRef(),
-                getter_AddRefs(tmpOutstanding));
+                sDNSListener, nullptr, oa, getter_AddRefs(tmpOutstanding));
             // Fetch ESNI keys if needed.
-            if (NS_SUCCEEDED(rv) && sEsniEnabled && isHttps) {
+            if (NS_SUCCEEDED(rv) &&
+                StaticPrefs::network_security_esni_enabled() && isHttps) {
               nsAutoCString esniHost;
               esniHost.Append("_esni.");
               esniHost.Append(hostName);
               sDNSService->AsyncResolveByTypeNative(
                   esniHost, nsIDNSService::RESOLVE_TYPE_TXT,
                   mEntries[mTail].mFlags | nsIDNSService::RESOLVE_SPECULATE,
-                  sDNSListener, nullptr,
-                  element->NodePrincipal()->OriginAttributesRef(),
-                  getter_AddRefs(tmpOutstanding));
+                  sDNSListener, nullptr, oa, getter_AddRefs(tmpOutstanding));
             }
             // Tell link that deferred prefetch was requested
             if (NS_SUCCEEDED(rv)) link->OnDNSPrefetchRequested();
